@@ -2,14 +2,14 @@ package EZDBI;
 use DBI;
 use strict;
 use Carp;
-use vars ('$E', '@EXPORT', '$VERSION', '$MAX_STH');
+use vars qw'$E @EXPORT @EXPORT_OK $VERSION $MAX_STH $BINDABLE';
 require 5;
 
 my $DBH;
 *E = \$DBI::errstr;
 my $sth_cache;   # string to statement handle cache
 my $sth_cacheA;  # oldest first (LRU)  handle order
-$VERSION = 0.13;
+$VERSION = 0.14;
 
 # Note that this package does NOT inherit from Exporter
 @EXPORT = qw(Connect Delete Disconnect Insert Select Sql Update Use);
@@ -18,6 +18,10 @@ sub import {
   my ($package, %parms) = @_;
   my $caller = caller;
 
+  #Load constants for bindable values
+  DBI->export_to_level(1, undef, ':sql_types')
+    if $BINDABLE = $parms{sql_types} || $parms{':sql_types'};
+
   #This is per database handle
   $MAX_STH = $parms{maxQuery} || 10;
 
@@ -63,7 +67,7 @@ sub Connect {
 
 sub Delete {
   my ($str, @args) = @_;
-  my $sth = _substitute('Delete', $str, scalar @args);
+  my $sth = _substitute('Delete', $str, \@args);
   my $rc;
   unless ($rc = $sth->execute(@args)) {
     croak "Delete failed: $E";
@@ -92,7 +96,7 @@ sub Insert {
     @args = @hash{@cols};
   }
 
-  my $sth = _substitute('Insert', $str, scalar @args);
+  my $sth = _substitute('Insert', $str, \@args);
   my $rc;
   unless ($rc = $sth->execute(@args)) {
     croak "Insert failed: $E";
@@ -112,7 +116,7 @@ sub Select {
 
   croak "Select in void context" unless defined wantarray;
 
-  my $sth = _substitute('Select', $str, scalar @args);
+  my $sth = _substitute('Select', $str, \@args);
   unless ($sth->execute(@args)) {
     croak "Select failed: $E";
   }
@@ -171,7 +175,7 @@ sub Update {
     @args = (@hash{@cols}, @args);
   }
 
-  my $sth = _substitute('Update', $str, scalar @args);
+  my $sth = _substitute('Update', $str, \@args);
   my $rc;
   unless ($rc = $sth->execute(@args)) {
     croak "Update failed: $E";
@@ -214,24 +218,25 @@ sub _parseIni{
 sub _substitute {
   defined($DBH) || croak "Not connected to a database";
   my($function, $str, $args) = @_;
+  my $Nargs = scalar @{$args};
 
   if( $function eq 'Insert' ){
-    my $list = join ',' , (('?') x $args);
+    my $list = join ',' , (('?') x $Nargs);
     unless( $str =~ s/\?\?L|\(\s*\?\?L\s*\)/($list)/ ){
       if( $str =~ /\bvalues\b/i ){
         $str .= "($list)" unless $str =~ /\)\s*$/;
       }
-      elsif( $args ){
+      elsif( $Nargs ){
         $str .= " values ($list)";
       }
     }
   }
 
   my $subct = $str =~ tr/?/?/;
-  if( $subct > $args ){
+  if( $subct > $Nargs ){
     croak "Not enough arguments for $function ($subct required)";
   }
-  elsif( $subct < $args ){
+  elsif( $subct < $Nargs ){
     croak "Too many arguments for $function ($subct required)";
   }
 
@@ -248,11 +253,18 @@ sub _substitute {
     if( scalar @{$sth_cacheA->{$DBH}} >= $MAX_STH -1 ){
       delete(@{$sth_cache->{$DBH}}{splice(@{$sth_cacheA->{$DBH}},0,$MAX_STH/3)});
     }
-    
+
     # prepare new handle
     $sth = $DBH->prepare("$function $str");
     croak "Couldn't prepare query for '$function $str': $E; aborting" unless $sth;
-    
+    #XXX BINDING
+    foreach(my $i=0; $BINDABLE && $i<$Nargs; $i++){
+      if( ref($args->[$i]) eq 'ARRAY' ){
+	$sth->bind_param($i+1, undef, $args->[$i]->[1]);
+	$args->[$i] = $args->[$i]->[0];
+      }
+    }
+
     # install new handle in cache
     $sth_cache->{$DBH}->{$str} = $sth;
     push(@{$sth_cacheA->{$DBH}}, $str);
@@ -599,6 +611,21 @@ allows you to do this more efficiently.
 Rather this is most appropriate when connections to databases on seperate
 machines need to be maintained.
 
+=head2 Binding
+
+If you need to bind your parameters to a specific data type, perhaps because
+Microsoft ODBC is misinterpeting your binary data as VARCHAR and storing your
+data as UCS-2LE (a.k.a extra nulls), EZDBI can handle this as well. Simply
+import the data type constants (L<DBI/"DBI Constants">), and pass any values
+you wish to be bound/cast as tuples like so:
+
+  use EZDBI sql_types=>1;
+  ...
+  Update('Foo SET Baz=?, Fred=?', ['Qux'=>SQL_LONGVARBINARY], 'Flintstone');
+
+Note: It is senseless to change the binding type of a column between
+invocations, and doing so may be hazardous to your health.
+
 =head1 ERRORS
 
 If there's an error, B<EZDBI> prints a (hopefully explanatory) message
@@ -650,9 +677,7 @@ and unsupported and may go away in a future release. Inquire within.
 
 =head1 BUGS
 
-This is ALPHA software.
-There may be bugs.
-The interface may change.
+There may be bugs. The interface may change.
 
 =head1 AUTHORS
 
@@ -680,19 +705,19 @@ Gavin Estey
 =head1 COPYRIGHT
 
   EZDBI - Easy Perl interface to SQL databases
-  Portions Copyright (C) 2002  Jerrad Pierce
-           Copyright (C) 2001  Mark Jason Dominus
-  
+  Portions Copyright (C) 2016, 2002  Jerrad Pierce
+           Copyright (C) 2001        Mark Jason Dominus
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
-  
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.
-  
+
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
